# generate-dependencies.sh
# Version 0.0
# "graphein" CircuitousRoot.com / lemur.com Tools
# See http://www.CircuitousRoot.com/graphein/ for documentation.
#
# Copyright 2006, 2008 by David M. MacMillan and Rollande Krandall.
#
# This work is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#    
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.
# 59 Temple Place, Suite 330
# Boston, MA  02111-1307
# USA
# 
# Circuitous Root is a registered trademark of David M. MacMillan and 
# Rollande Krandall
# 
# Presented originally by Circuitous Root (R). http://www.CircuitousRoot.com/


ls *.tei > /dev/null 2> /dev/null
if [ $? -eq 0 ]; then
   for f in `ls *.tei`; do
      fbaselength=`expr index $f "."`
      fbaselength=`expr $fbaselength - 1`
      fbase=`expr substr $f 1 $fbaselength`
      echo -n "`basename $fbase`.tei: "
      awk -f generate-dependencies.awk $f;
      echo 
      echo "	touch `basename $fbase`.tei "
   done
fi
echo

# image scaling
# loop over all desired scale factors (SF) (0 to max; max in 1st parameter)
#    loop over all PNGs (masters always PNG; output will be JPG)
#       generate make rule to make it using ImageMagick convert
#       Formula for scale width: 2^N * 100
#          where N is (currentSF) - (image master SF)
#             Example (master SF as big as possible, at SF4)
#                         0 - 4 = -4, so 2^-4 * 100 =   6.25 %
#                         1 - 4 = -3, so 2^-3 * 100 =  12.5  %
#                         2 - 4 = -2, so 2^-2 * 100 =  25    %
#                         3 - 4 = -1, so 2^-1 * 100 =  50    %
#                         4 - 4 =  0, so 2^0  * 100 = 100    %
#                         (note: doing a 100% convert is inefficient,
#                                but not incorrect)
#             Example (master SF in middle at SF2)
#                         0 - 2 = -2, so 2^-2 * 100 =  25 %
#                         1 - 2 = -1, so 2^-1 * 100 =  50 %
#                         2 - 2 =  0, so 2^0  * 100 = 100 %
#                         3 - 2 =  1, so 2^1  * 100 = 200 %
#                         4 - 2 =  2, so 2^2  * 100 = 400 %
#          Every so often I *almost* get the real mathematician's sense
#          of how beautiful it all is when the numbers work out.
#       As it is a percentage scaling, 
#          the original image width doesn't matter
#
# Put it all in a bash function so it can be reused for relative scaling
#
# Never underestimate the power of standard utilities (bash, echo, awk, bc).  
# Whew. :-)
# But also, never assume this is secure! Do NOT run this program in anything
# other than an absolutely trusted context.
#       
cd images-mastered-for-scaling;

currentSF=0
echo -n "images-scaled.dep: "
while [ $currentSF -le $1 ]; do
   for image in `ls *-sf?.png`; do
      echo "\\"
      echo -n "`echo $image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg"
   done
   currentSF=`expr $currentSF + 1`
done
echo
echo -n "	touch images-scaled.dep"
echo
echo

currentSF=0
while [ $currentSF -le $1 ]; do
   for image in `ls *-sf?.png`; do
      masterSF=`echo $image | awk '{ print substr($1,index($1,".")-1,1) }'`
         echo "`echo $image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg: images-mastered-for-scaling/$image"
         echo "	(cd images-mastered-for-scaling; \\"
echo "	pwd; \\"
         echo "	cp -f $image trash.png; \\"
         echo "	convert trash.png -resize `echo "2^($currentSF-$masterSF) * 100" | bc -l`% `echo $image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg; \\"
         echo "	rm -f trash.png; \\"
         echo "	mv -f `echo $image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg ..; \\"
         echo "	cd .. )"
         echo
         echo
         echo "images-mastered-for-scaling/$image:"
         echo
   done;
   currentSF=`expr $currentSF + 1`
done
cd ..



# private image handling was hacked in a bit later, 2010-01-18.
# The idea is that private information of all types should be in its
# own subtree, so that it is easier not to publish it.  This is especially
# important when it consists of local copies of works in copyright.
#
# To make things both private and easily excluded from the public 
# release, do three things:
# 1. Mark private elements (p, ab, etc.) as n="private" as usual.
#    This will keep them from being generated on make.
# 2. Put all private data (e.g., PDF files of documents) in
#    the ./private subdirectory.  Reference them there; do not copy them
#    to the main directory.  Since the private copy is local, they'll be
#    readable from the local filesystem.
# 3. Put images mastered for linking icons for private data in
#    ./private/images-mastered-for-scaling
#    and create link-to- links as usual there.
#    Reference these as "private/link-to-..." in the markup.
# 
# The code here for generating links is not directly executed.
# The code here is output to stdout and redirected into "dependencies.dep"
# dependencies.dep is in turn included in the overall makefile.
# The make target images-scaled-private.dep (new here) causes the dependencies
# so generated to be executed.  The execution of these dependencies
# traverses private/images-mastered-for-scaling and creates the
# scaled versions. These are moved up to ./private 

cd private/images-mastered-for-scaling;

currentSF=0
echo -n "images-scaled-private.dep: "
while [ $currentSF -le $1 ]; do
   for image in `ls *-sf?.png`; do
      echo "\\"
      echo -n "`echo private/$image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg"
   done
   currentSF=`expr $currentSF + 1`
done
echo
echo -n "	touch images-scaled-private.dep"
echo
echo

currentSF=0
while [ $currentSF -le $1 ]; do
   for image in `ls *-sf?.png`; do
      masterSF=`echo private/$image | awk '{ print substr($1,index($1,".")-1,1) }'`
         echo "`echo private/$image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg: private/images-mastered-for-scaling/$image"
         echo "	(cd private/images-mastered-for-scaling; \\"
echo "	pwd; \\"
         echo "	cp -f $image trash.png; \\"
         echo "	convert trash.png -resize `echo "2^($currentSF-$masterSF) * 100" | bc -l`% `echo $image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg; \\"
         echo "	rm -f trash.png; \\"
         echo "	mv -f `echo $image | awk '{ print substr($1,0,index($1,".")-2) }'`$currentSF.jpg ..; \\"
         echo "	cd .. )"
         echo
         echo
         echo "private/images-mastered-for-scaling/$image:"
         echo
   done;
   currentSF=`expr $currentSF + 1`
done
cd ../..
