<?xml version="1.0" encoding="utf-8" ?>
<!-- 
graphein-tohtml.xsl
Version 0.3
"graphein" Tools
See http://www.CircuitousRoot.com/graphein/ for documentation.

Copyright 2006-2010 by David M. MacMillan and Rollande Krandall.

This work is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
   
You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.
59 Temple Place, Suite 330
Boston, MA  02111-1307
USA

Circuitous Root is a registered trademark of David M. MacMillan and 
Rollande Krandall

Presented originally by Circuitous Root (R). http://www.CircuitousRoot.com/


XSLT to Transform TEI to XHTML
Requires:    A CSS stylesheet
             TEI P5 encoded source
                Attribute convensions as discussed in the graphein documents
             XSLT 1.1 compliant processor
Parameters:  required: base name of a CSS stylesheet
Outputs:     XHTML 1.0
Tested with: Saxon 6.5.3
-->

<xsl:stylesheet 
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:tei="http://www.tei-c.org/ns/1.0"
   version="2.0">

<!-- ========== -->
<!-- Parameters -->
<!-- ========== -->
<xsl:param name="own-basename" />
<xsl:param name="css-basename" />
<xsl:param name="scale-factor" />
<xsl:param name="location" />
<xsl:param name="depth-in-hierarchy" />
<xsl:param name="depth-in-category" />
<xsl:param name="path-to-category-top" />
<xsl:param name="path-to-current" />
<xsl:param name="public-or-private" />

<!-- ================= -->
<!-- XSL Configuration -->
<!-- ================= -->
<!-- note: Until 0.3 I used "xml" here.  But the xml output method causes
           Saxon's serializer to output empty tags as standalone tags
           (ending in /> ) rather than as separate start and end tags.
           There is a bug in Firefox such that a standalone "name" or "id"
           anchor will respond to the "hover" CSS just like a clickable link.
           So compensate for that bug here by specifying xhtml.
-->
<xsl:output method="xhtml"
            encoding="utf-8" />

<!-- Suppress inter-tag whitespace in output. -->
<!-- (Do output prettyprinting explicitly, adding #xA (linefeed) entities
     and spaces explicitly in xsl:text.) -->
<xsl:strip-space elements="*" />
<!-- note: opposite is xsl:preserve-space -->


<!-- ================== -->            <!--  /  -->
<!-- root and TEI nodes -->            <!--  |  -->
<!-- ================== -->            <!-- TEI -->
<xsl:template match="/">  <!-- yes, this is redundant -->
   <xsl:apply-templates/>
</xsl:template>

<xsl:template match="tei:TEI">
   <xsl:text>&#xA;</xsl:text>
   <html>
      <xsl:apply-templates/>
   <xsl:text>&#xA;</xsl:text>
   </html>
</xsl:template>

<!-- ============= -->                 <!-- TEI              -->
<!-- TEI teiHeader -->                 <!--  |.... teiHeader -->
<!-- ============= -->                 <!--  |.... text      -->
<xsl:template match="tei:teiHeader">
   <xsl:text>&#xA;</xsl:text>
   <head>
      <xsl:text>&#xA;</xsl:text>
      <title>
         <xsl:value-of select="//tei:titleStmt" />
      </title>
      <xsl:text>&#xA;</xsl:text>
      <link>

      <xsl:variable name="home-relative-url">
         <xsl:call-template name="linkup">
            <xsl:with-param name="levels" select="$depth-in-hierarchy"/>
         </xsl:call-template>
      </xsl:variable>

         <xsl:attribute name="rel">stylesheet</xsl:attribute>
         <xsl:attribute name="type">text/css</xsl:attribute>
         <xsl:attribute name="media">screen</xsl:attribute>
         <xsl:attribute name="title">
            <xsl:value-of select="$css-basename" />
         </xsl:attribute>
         <xsl:attribute name="href">
            <xsl:value-of select="$home-relative-url"/>
            <xsl:value-of select="$css-basename"/>
            <xsl:text>.css</xsl:text>
         </xsl:attribute>
      </link>

   <xsl:text>&#xA;</xsl:text>
   </head>
</xsl:template>

<!-- suppress other teiHeader stuff -->
<xsl:template match="tei:fileDesc"></xsl:template>
<xsl:template match="tei:editionStmt"></xsl:template>
<xsl:template match="tei:publicationStmt"></xsl:template>
<xsl:template match="tei:sourceDesc"></xsl:template>
<xsl:template match="tei:revisionDesc"></xsl:template>

<!-- ============= -->                 <!-- text ......front -->
<!-- TEI text Node -->                 <!--        |...body  -->
<!-- ============= -->                 <!--        |...back  -->
<xsl:template match="tei:text">
   <xsl:apply-templates/>
</xsl:template>

<xsl:template match="tei:body">
   <xsl:text>&#xA;</xsl:text>
   <body>
      <xsl:apply-templates/>
   </body>
</xsl:template>

<!-- ================ -->
<!-- TEI Front Matter -->
<!-- ================ -->
<xsl:template match="tei:front">
   <xsl:apply-templates/>
</xsl:template>

<xsl:template match="tei:docTitle">
   <div class="broadside">
   <xsl:apply-templates/>
   </div>
</xsl:template>

<xsl:template match="tei:titlePart[@type='main']">
   <xsl:text>&#xA;</xsl:text>
   <xsl:text>&#xA;</xsl:text>
   <h1 class="main">
      <xsl:value-of select="." />
   </h1>
</xsl:template>

<xsl:template match="tei:titlePart[@type='sub']">
   <xsl:text>&#xA;</xsl:text>
   <xsl:text>&#xA;</xsl:text>
   <h1 class="sub">
      <xsl:value-of select="." />
   </h1>
</xsl:template>

<!-- Grab the contents of the teiHeader publicationStmt availability section,
     format it for the "legal matters" div, and put it whereever this
     template is called from (right before the close of the body,
     as it will happen. -->
<xsl:template name="legal-matters">
   <xsl:text>&#xA;</xsl:text>
   <div class="broadside">
   <div class="legal-matters">
      <xsl:apply-templates select="//tei:availability"/>
   <xsl:text>&#xA;</xsl:text>
   </div></div>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-item-freeform']">
   <xsl:text>&#xA;</xsl:text>
   <p>
      <xsl:apply-templates/>
   </p>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-item-pd']">
   <xsl:text>&#xA;</xsl:text>
   <p>
      <xsl:apply-templates/>
   </p>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-item-regtm']">
   <xsl:text>&#xA;</xsl:text>
   <p>
      <xsl:apply-templates/>
   </p>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-license-ccbysa']">
   <xsl:text>&#xA;</xsl:text>
   <p>This work is licensed under the 
      Creative Commons "Attribution - ShareAlike" license.
      See 
      <a href="http://creativecommons.org/licenses/by-sa/3.0/">
      http://creativecommons.org/licenses/by-sa/3.0/</a>
      for its terms.</p>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-license-gfdl']">
   <xsl:text>&#xA;</xsl:text>
   <p>Permission is granted to copy, distribute and/or modify this document
    under the terms of the GNU Free Documentation License, Version 1.2
    or any later version published by the Free Software Foundation;
    with no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts.
    A copy of the license is included in the section entitled 
    "<a href="gfdl">GNU Free Documentation License</a>".</p>

</xsl:template>

<xsl:template match="tei:ab[@type='legal-license-gfdl-ccbysa']">
   <xsl:text>&#xA;</xsl:text>
   <p>This work is licensed under either of
      two licenses, at your option:</p>

   <p>(A) Permission is granted to copy, distribute and/or modify this document
    under the terms of the GNU Free Documentation License, Version 1.2
    or any later version published by the Free Software Foundation;
    with no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts.
    A copy of the license is included in the section entitled 
    "<a href="gfdl">GNU Free Documentation License</a>".</p>

   <p>(B) This work is licensed under the 
      Creative Commons "Attribution - ShareAlike" license.
      See 
      <a href="http://creativecommons.org/licenses/by-sa/3.0/">
      http://creativecommons.org/licenses/by-sa/3.0/</a>
      for its terms.</p>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-license-gpl-gfdl']">
   <xsl:text>&#xA;</xsl:text>
   <p>The computer program(s) contained in this work are free software;
      you can redistribute them and/or modify
      them under the terms of the 
      <a href="gpl">GNU General Public License</a> as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.</p>

   <p>These programs are distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.</p>

   <p>You should have received a copy of the GNU General Public License
      along with these programs; if not, write to the
      Free Software Foundation, Inc.
      59 Temple Place, Suite 330
      Boston, MA  02111-1307
      USA</p>


   <p>Permission is granted to copy, distribute and/or modify this document
    under the terms of the GNU Free Documentation License, Version 1.2
    or any later version published by the Free Software Foundation;
    with no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts.
    A copy of the license is included in the section entitled 
    "<a href="gfdl">GNU Free Documentation License</a>".</p>
</xsl:template>

<xsl:template match="tei:ab[@type='legal-license-gpl-gfdl-ccbysa']">
   <xsl:text>&#xA;</xsl:text>
   <p>The computer program(s) contained in this work are free software;
      you can redistribute them and/or modify
      them under the terms of the 
      <a href="gpl">GNU General Public License</a> as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.</p>

   <p>These programs are distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.</p>

   <p>You should have received a copy of the GNU General Public License
      along with these programs; if not, write to the
      Free Software Foundation, Inc.
      59 Temple Place, Suite 330
      Boston, MA  02111-1307
      USA</p>

   <p>This work considered as a document is licensed under either of
      two licenses, at your option:</p>

   <p>(A) Permission is granted to copy, distribute and/or modify this document
    under the terms of the GNU Free Documentation License, Version 1.2
    or any later version published by the Free Software Foundation;
    with no Invariant Sections, no Front-Cover Texts, and no Back-Cover Texts.
    A copy of the license is included in the section entitled 
    "<a href="gfdl">GNU Free Documentation License</a>".</p>

   <p>(B) This work is licensed under the 
      Creative Commons "Attribution - ShareAlike" license.
      See 
      <a href="http://creativecommons.org/licenses/by-sa/3.0/">
      http://creativecommons.org/licenses/by-sa/3.0/</a>
      for its terms.</p>
</xsl:template>

<!-- These trademark recognition statements are appropriate for the
     author; that's one advantage, at least, of writing your own system.  -->
<xsl:template match="tei:ab[@type='legal-trademark-circuitousroot']">
   <xsl:text>&#xA;</xsl:text>
   <p>Circuitous Root is a Registered Trademark 
      of David M. MacMillan and Rollande Krandall.</p>
</xsl:template>
<xsl:template match="tei:ab[@type='legal-trademark-lemurcom']">
   <xsl:text>&#xA;</xsl:text>
   <p>lemur.com is a Trademark 
      of Rollande Krandall and David M. MacMillan.</p>
</xsl:template>
<xsl:template match="tei:ab[@type='legal-trademark-singinglemur']">
   <xsl:text>&#xA;</xsl:text>
   <p>Singing Lemur is a Registered Trademark 
      of Rollande Krandall.</p>
</xsl:template>

<xsl:template match="tei:ab[@type='presented-by']">
   <xsl:text>&#xA;</xsl:text>
   <p>Presented originally by
            <xsl:value-of select="." />
<!--
      <a>
      <xsl:attribute name="href">
            <xsl:value-of select="." />
      </xsl:attribute>
      </a>
-->
   </p>
</xsl:template>

<!-- note: 'ab type="code" is block-level;
           for phrase-level code examples, use "code" -->
<xsl:template match="tei:ab[@type='code']">
   <xsl:text>&#xA;</xsl:text>
   <pre>
      <xsl:apply-templates/>
   </pre>
</xsl:template>

<xsl:template name="pd-disclaimers">
   <strong>
   <a href="http://www.circuitousroot.com/pd-disclaimers.html">Important 
   disclaimers of warranty and liability in the presentation 
   of public domain material</a>.
   </strong>
</xsl:template>

<!-- ======== -->
<!-- TEI Body -->
<!-- ======== -->


<!-- HACK: xml:id tags are turned into "a name" so that the #fragment
     linking works.
     This should be done for all block-level elements, but for now I've
     just hacked it into AB, DIV, LIST, ITEM, and P -->
<!-- SUBHACK: it should be possible to use a self-contained anchor tag,
     but without separate start and end tags some versions of Firefox,
     at least, highlight the entity on mouse-over. -->

<!-- on the xsl:number, count only chapters; this starts the counting
     after any preliminary "broadside-divider" images -->
<xsl:template match="tei:div[@type='chapter']">
   <xsl:text>&#xA;</xsl:text>
   <xsl:text>&#xA;</xsl:text>
   <div class="chapter">
   <h2>
   <a name="{generate-id(.)}">
   <xsl:number level="multiple" count="tei:div[@type='chapter']" format="1.1. "/>
   <xsl:value-of select="tei:head" />
   </a>
   </h2>
   <a><xsl:attribute name="id">
      <xsl:value-of select="@xml:id" /></xsl:attribute>
      <xsl:text> </xsl:text>
      </a>
   <xsl:apply-templates />
   </div>
</xsl:template>


<!-- on the xsl:number, count only chapters and sections; 
     this continues the counting properly after any preliminary 
     "broadside-divider" images -->
<xsl:template match="tei:div[@type='section']">
   <xsl:text>&#xA;</xsl:text>
   <xsl:text>&#xA;</xsl:text>
   <h2>
   <a name="{generate-id(.)}">
   <xsl:number level="multiple" count="tei:div[@type='section']|tei:div[@type='chapter']" format="1.1. "/>
   <xsl:value-of select="tei:head" />
   </a>
   </h2>
   <a><xsl:attribute name="id">
      <xsl:value-of select="@xml:id" /></xsl:attribute>
      <xsl:text> </xsl:text>
      </a>
   <xsl:apply-templates />
</xsl:template>

<xsl:template match="tei:div[@type='broadside']">
   <xsl:text>&#xA;</xsl:text>
   <xsl:text>&#xA;</xsl:text>
   <div class="broadside">
   <a><xsl:attribute name="id">
      <xsl:value-of select="@xml:id" /></xsl:attribute>
      </a>
   <xsl:apply-templates />
   </div>
</xsl:template>

<xsl:template match="tei:div[@type='broadside-divider']">
   <xsl:text>&#xA;</xsl:text>
   <xsl:text>&#xA;</xsl:text>
   <div class="broadside-divider">
   <a><xsl:attribute name="id">
      <xsl:value-of select="@xml:id" /></xsl:attribute>
      </a>
   <xsl:apply-templates />
   </div>
</xsl:template>

<!-- This is null because we pick up the traversal of tei:head
     in the tei:div[@type='section'] template, above. -->
<xsl:template match="tei:head"></xsl:template>


<!-- Match tei:bibl only if NOT in a citation -->
<xsl:template match="tei:bibl">
   <xsl:choose>
      <xsl:when test="ancestor::tei:cit">
<!--
      <xsl:when test="@n='citation'">
-->
         <xsl:text>&#xA;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
         <xsl:text>&#xA;</xsl:text>
         <p class="bibl">
         <xsl:attribute name="id">
              <xsl:value-of select="@xml:id" /></xsl:attribute>
         <xsl:apply-templates/>
         </p>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="tei:ab[@type='bibl-annotation']">
   <xsl:text>&#xA;</xsl:text>
   <p class="bibl-annotation">
      <xsl:apply-templates/>
   </p>
</xsl:template>


<!-- HACK: xml:id tags are turned into "a name" so that the #fragment
     linking works.
     This should be done for all block-level elements, but for now I've
     just hacked it into AB, DIV, LIST, ITEM, and P -->
<!-- SUBHACK: it should be possible to use a self-contained anchor tag,
     but without separate start and end tags some versions of Firefox,
     at least, highlight the entity on mouse-over. -->

<xsl:template match="tei:p">
   <xsl:choose>
      <!-- if it is a private p, -->
      <xsl:when test="@n='private'">
         <!-- and it is a private build, then build it -->
         <!--     and identify it as private, so I'll know -->
         <xsl:if test="$public-or-private = 'private'">
            <xsl:text>&#xA;</xsl:text>
            <p class="private">
            <a><xsl:attribute name="id">
               <xsl:value-of select="@xml:id" /></xsl:attribute>
               <xsl:text> </xsl:text>
               </a>
            <xsl:apply-templates/>
            </p>
         </xsl:if>
         <!-- but if it is a public build, then don't build it -->

      </xsl:when>
      <!-- if it is a non-private p, always build it -->
      <xsl:otherwise>
         <xsl:text>&#xA;</xsl:text>
         <p>
         <a><xsl:attribute name="id">
            <xsl:value-of select="@xml:id" /></xsl:attribute>
               <xsl:text> </xsl:text>
            </a>
         <xsl:apply-templates/>
         </p>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="tei:ab[@type='epigraph']">
   <xsl:text>&#xA;</xsl:text>
   <div class="epigraph">
   <xsl:apply-templates/>
   </div>
</xsl:template>

<!-- N.B.: (trick for CSS) output an empty, clear=both div as
     the last thing to ensure that by growing to include it
     (and it clears left and right) the enclosing CSS box will
     actually be as tall as the stuff it encloses. 
     The xsl:text tag forces Saxon, at least, to use an explicit
     closing tag (which shouldn't matter to the browser, but does). -->
<!-- HACK: xml:id tags are turned into "a name" so that the #fragment
     linking works.
     This should be done for all block-level elements, but for now I've
     just hacked it into AB, DIV, LIST, ITEM, and P -->
<!-- SUBHACK: it should be possible to use a self-contained anchor tag,
     but without separate start and end tags some versions of Firefox,
     at least, highlight the entity on mouse-over. -->
<xsl:template match="tei:ab[@type='abstract']">
   <xsl:choose>
      <!-- if it is a private ab, -->
      <xsl:when test="@n='private'">
         <!-- and it is a private build, then build it -->
         <!--     and identify it as private, so I'll know -->
         <xsl:if test="$public-or-private = 'private'">
            <xsl:text>&#xA;</xsl:text>
            <div class="abstract-private">
         <a><xsl:attribute name="id">
              <xsl:value-of select="@xml:id" /></xsl:attribute>
              <xsl:text> </xsl:text>
         </a>
            <xsl:apply-templates/>
            <div style="clear:both;"><xsl:text>
</xsl:text></div></div>
         </xsl:if>
         <!-- but if it is a public build, then don't build it -->

      </xsl:when>
      <!-- if it is a non-private ab, always build it -->
      <xsl:otherwise>
         <xsl:text>&#xA;</xsl:text>
         <div class="abstract">
         <a><xsl:attribute name="id">
              <xsl:value-of select="@xml:id" /></xsl:attribute>
              <xsl:text> </xsl:text>
         </a>
         <xsl:apply-templates/>
         <div style="clear:both;"><xsl:text>
</xsl:text></div></div>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>


<xsl:template match="tei:distinct[@type='abstract-title']">
   <xsl:text>&#xA;</xsl:text>
   <p class="abstract-title">
   <xsl:apply-templates/>
   </p>
</xsl:template>

<xsl:template match="tei:distinct[@type='abstract-p']">
   <xsl:text>&#xA;</xsl:text>
   <p class="abstract-p">
   <xsl:apply-templates/>
   </p>
</xsl:template>

<!-- 
NOTE: Previously I output <span>click...</span><br /> for this.
      That worked, but then I changed the output method to xhtml from
      xml to force the output of full opening-and-closing tags (to bypass
      a Firefox bug in hovering over link targets).
      BUT Firefox (2.6 at least) has a longstanding bug whereby it 
      emits not one but TWO newlines for <br></br>.  Sigh.

      So change this to <div>click...</div> with no <br />
      This requires changing main.css, too.
-->

<xsl:template match="tei:distinct[@type='click-prompt-go']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-go">[click image to go to page]</div>
</xsl:template>

<xsl:template match="tei:distinct[@type='click-prompt-go-blank']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-go">[click blank image to go to page]</div>
</xsl:template>

<xsl:template match="tei:distinct[@type='click-prompt-read']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-read">[click image to read]</div>
</xsl:template>

<xsl:template match="tei:distinct[@type='click-prompt-read-ia']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-read">[click image to read at The Internet Archive]</div>
</xsl:template>

<xsl:template match="tei:distinct[@type='click-prompt-read-blank']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-read">[click blank image to read]</div>
</xsl:template>

<xsl:template match="tei:distinct[@type='click-prompt-view']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-view">[click image to view larger]</div>
</xsl:template>

<xsl:template match="tei:distinct[@type='click-prompt-view-blank']">
   <xsl:text>&#xA;</xsl:text>
   <div class="click-prompt-view">[click blank image to view larger]</div>
</xsl:template>

<xsl:template match="tei:emph">
   <xsl:text>&#xA;</xsl:text>
   <i>
   <xsl:apply-templates/>
   </i>
</xsl:template>

<xsl:template match="tei:hi[@rend='italic']">
   <xsl:text>&#xA;</xsl:text>
   <i>
   <xsl:apply-templates/>
   </i>
</xsl:template>

<!-- +++++++++++++ -->
<!-- "boilerplate" -->
<!-- +++++++++++++ -->
<xsl:template match="tei:ab[@type='boilerplate-about-the-images']">
   <xsl:text>&#xA;</xsl:text>
   <p>All of the images here are in the public domain in the USA,
   with sources as indicated.</p>
   <p>Often the originals are too large to present online.
   Click (or right-click to download) on each image for a larger
   (if not as large as possible) version.</p>
   <p>The images which serve as links to other documents
   on this site
   are described on the "About the Images" pages for
   those linked documents.</p>
</xsl:template>

<!-- For "grapheinlink" links, the "target" attribute specifies only
     the base name of the link.  
     If it contains a "#", then this attribute's value must be split
     into filename#location-in-file.
     Append a scale factor suffix and ".html" to the filename,
     then (if present), #location-in-file 
     The alternative is to code the location-in-file separately in the TEI,
     but the TEI doesn't really accomodate this
     (the "target" attribute takes a URI, basically, which is what this is;
     the alternative cRef (canonical ref) attribute doesn't quite fit).
     The XPath substring-before() and substring-after() make this almost
     too easy (the only trick is to realize that if there is no # then
     substring-before(), logically, returns an empty string).
     Note: The part after the # is called the "fragment identifier"
           This is defined in RFC 2396
           "The format and interpretation of fragment identifiers is
           dependent on the media type (RFC2046) of the retrieval result."
           So I think there's no reason it can't be null, which makes
           it possible always to append # here, even when substring-after()
           returns an empty string.
-->
<xsl:template match="tei:ref[@n='grapheinlink']">
   <xsl:text>&#xA;</xsl:text>
   <a>
<!-- HACK:
For now, just force everything to link to "xxx.html" rather than 
         xxx.html for sf0 and xxx-N.html for sf >=1
-->
   <xsl:if test="true() = contains(@target,'#')">
      <xsl:attribute name="href">
           <xsl:value-of select="substring-before(@target,'#')" />.html#<xsl:value-of select="substring-after(@target,'#')" /></xsl:attribute>
   </xsl:if>
   <xsl:if test="true() != contains(@target,'#')">
      <xsl:attribute name="href">
           <xsl:value-of select="@target" />.html</xsl:attribute>
   </xsl:if>
<!--
   <xsl:if test="true() = contains(@target,'#')">
      <xsl:attribute name="href">
           <xsl:value-of select="substring-before(@target,'#')" />-<xsl:value-of select="$scale-factor" />.html#<xsl:value-of select="substring-after(@target,'#')" /></xsl:attribute>
   </xsl:if>
   <xsl:if test="true() != contains(@target,'#')">
      <xsl:attribute name="href">
           <xsl:value-of select="@target" />-<xsl:value-of select="$scale-factor" />.html</xsl:attribute>
   </xsl:if>
-->
   <xsl:apply-templates/>
   </a>
</xsl:template>

<xsl:template match="tei:ref[@n='image']">
   <xsl:text>&#xA;</xsl:text>
   <a>
   <xsl:attribute name="href">/images/<xsl:value-of select="$path-to-current" /><xsl:value-of select="@target" /></xsl:attribute>
   <xsl:apply-templates/>
   </a>
</xsl:template>

<!-- For non-"grapheinlink" links and non-"image" links, the "target" 
     attribute specifies
     the entire link filename, including any ".jpg" or ".png"
     suffixes (for local images) or ".html" suffixes (probably for
     offsite links. -->
<xsl:template match="tei:ref">
   <xsl:text>&#xA;</xsl:text>
   <a>
   <xsl:attribute name="href">
        <xsl:value-of select="@target" /></xsl:attribute>
   <xsl:apply-templates/>
   </a>
</xsl:template>


<!-- HACK: xml:id tags are turned into "a name" so that the #fragment
     linking works.
     This should be done for all block-level elements, but for now I've
     just hacked it into AB, DIV, LIST, ITEM, CELL, and P -->
<!-- SUBHACK: it should be possible to use a self-contained anchor tag,
     but without separate start and end tags some versions of Firefox,
     at least, highlight the entity on mouse-over. -->
<!--
NOTE: I've just hacked "private" handling into LIST (see below),
      but I'm not convinced yet that I did it correctly.  I'll leave
      the old code in here for now.
<xsl:template match="tei:list">
   <xsl:text>&#xA;</xsl:text>
   <a><xsl:attribute name="id">
      <xsl:value-of select="@xml:id" /></xsl:attribute>
      <xsl:text> </xsl:text>
      </a>
   <ul>
   <xsl:apply-templates/>
   </ul>
</xsl:template>
-->

<xsl:template match="tei:list">
   <xsl:choose>
      <!-- if it is a private list, -->
      <xsl:when test="@n='private'">
         <!-- and it is a private build, then build it -->
         <!--     and identify it as private, so I'll know -->
         <xsl:if test="$public-or-private = 'private'">
            <xsl:text>&#xA;</xsl:text>
            <ul class="private">
            <a><xsl:attribute name="id">
               <xsl:value-of select="@xml:id" /></xsl:attribute>
               <xsl:text> </xsl:text>
               </a>
            <xsl:apply-templates/>
            </ul>
         </xsl:if>
         <!-- but if it is a public build, then don't build it -->

      </xsl:when>
      <!-- if it is a non-private p, always build it -->
      <xsl:otherwise>
         <xsl:text>&#xA;</xsl:text>
         <ul>
         <a><xsl:attribute name="id">
            <xsl:value-of select="@xml:id" /></xsl:attribute>
               <xsl:text> </xsl:text>
            </a>
         <xsl:apply-templates/>
         </ul>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>


<!-- HACK: xml:id tags are turned into "a name" so that the #fragment
     linking works.
     This should be done for all block-level elements, but for now I've
     just hacked it into AB, DIV, LIST, ITEM, CELL, and P -->
<!-- SUBHACK: it should be possible to use a self-contained anchor tag,
     but without separate start and end tags some versions of Firefox,
     at least, highlight the entity on mouse-over. -->
<xsl:template match="tei:item">
   <xsl:choose>
      <!-- a list ITEM may be one of three things:
           - private
           - notready, but public
           - public
      -->
      <!-- if it is a private list ITEM, -->
      <xsl:when test="@n='private'">
         <!-- and it is a private build, then build it -->
         <!--     and identify it as private, so I'll know -->
         <xsl:if test="$public-or-private = 'private'">
            <xsl:text>&#xA;</xsl:text>
            <a><xsl:attribute name="id">
               <xsl:value-of select="@xml:id" /></xsl:attribute>
               <xsl:text> </xsl:text>
               </a>
            <li class="private"><xsl:apply-templates/></li>
         </xsl:if>
         <!-- but if it is a public build, then don't build it -->
      </xsl:when>

      <!-- if it is a notready, but public ITEM,
           then build it and identify it as "notready"
           (right now the CSS draws a line through it,
           but that's the CSS, not the XSL here). -->
      <xsl:when test="@n='notready'">
         <xsl:text>&#xA;</xsl:text>
         <a><xsl:attribute name="id">
            <xsl:value-of select="@xml:id" /></xsl:attribute>
            <xsl:text> </xsl:text>
            </a>
         <li class="notready"><xsl:apply-templates/></li>
      </xsl:when>

      <!-- if it is a non-private item, always just build it -->
      <xsl:otherwise>
         <xsl:text>&#xA;</xsl:text>
         <a><xsl:attribute name="id">
            <xsl:value-of select="@xml:id" /></xsl:attribute>
            <xsl:text> </xsl:text>
            </a>
         <li><xsl:apply-templates/></li>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="tei:table">
   <xsl:text>&#xA;</xsl:text>
   <table>
   <xsl:apply-templates/>
   </table>
</xsl:template>

<xsl:template match="tei:row">
   <xsl:text>&#xA;</xsl:text>
   <tr>
   <xsl:apply-templates/>
   </tr>
</xsl:template>

<!-- Implement the anchor for CELL rather than ROW because CELL turns in to
     TD (within which HTML A is valid), while ROW becomes TR (within which
     HTML A is not valid). -->
<xsl:template match="tei:cell">
   <xsl:text>&#xA;</xsl:text>
   <td>
   <a><xsl:attribute name="id">
      <xsl:value-of select="@xml:id" /></xsl:attribute>
      <xsl:text> </xsl:text>
      </a>
   <xsl:apply-templates/>
   </td>
</xsl:template>

<xsl:template match="tei:cit">
   <xsl:text>&#xA;</xsl:text>
   <xsl:apply-templates/>
</xsl:template>

<xsl:template match="tei:quote">
   <xsl:text>&#xA;</xsl:text>
   <div class="bc">
   <xsl:apply-templates/>
        -  <xsl:value-of select="../tei:bibl/tei:author" />.
       <cite><xsl:value-of select="../tei:bibl/tei:title" /></cite>.
   </div>
</xsl:template>

<xsl:template match="tei:l">
   <xsl:text>&#xA;</xsl:text>
   <xsl:apply-templates/>
   <br />
</xsl:template>

<xsl:template match="tei:title">
   <i><xsl:apply-templates/></i>
</xsl:template>


<!-- Lexical Entries -->

<!-- 1. Processing wordlist forms -->
<xsl:template match="tei:note[@type='wordlist-headword']">
   <xsl:if test="not(ancestor::tei:entry)">
      <xsl:text>&#xA;</xsl:text>
               <xsl:variable name="category-relative-url">
               <xsl:call-template name="linkup-minus-one">
                  <xsl:with-param name="levels" select="$depth-in-category"/>
               </xsl:call-template>
            </xsl:variable>
      <a>
      <!-- Remember: the context node (.) here has been extracted from
           the target document and is now embedded in this document.
           Its parent here is not the same as in the target document. -->
      <xsl:attribute name="href"><xsl:value-of select="$category-relative-url" />../lexicon/index.html#<xsl:value-of select="escape-html-uri(@xml:id)" /></xsl:attribute>
         <span class="wordlist-headword"><xsl:value-of select="." /></span>,
      </a>
   </xsl:if>
</xsl:template>
<xsl:template match="tei:note[@type='wordlist-sense']">
   <xsl:if test="not(ancestor::tei:entry)">
      <xsl:text>&#xA;</xsl:text>
      <span class="wordlist-sense"> "<xsl:value-of select="." />"</span>
   </xsl:if>
</xsl:template>

<!-- 2. Processing full lexical entries -->

<!-- The xml:id of the ENTRY element is the first part of the
     xml:id of its "wordlist-headword" note; append "-wordlist-headword"
     to this to get a reference to that (they cannot simply be encoded
     as the same, as all xml:id values must be unique; the note's
     xml:id must be encoded as it will be XIncluded where the
     full entry is not).  -->
<xsl:template match="tei:entry">
   <xsl:text>&#xA;</xsl:text>
   <ab>
   <xsl:attribute name="id"><xsl:value-of select="escape-html-uri(@xml:id)" />-wordlist-headword</xsl:attribute>
   <xsl:attribute name="class">lexicon-entry</xsl:attribute>
      <xsl:apply-templates/>
   </ab>
</xsl:template>

<xsl:template match="tei:form[@type='simple']">
   <xsl:apply-templates/><xsl:text>.&#x20;</xsl:text>
</xsl:template>

<xsl:template match="tei:form[@type='inflection']">
</xsl:template>
<xsl:template match="tei:gramGrp">
</xsl:template>
<xsl:template match="tei:sense[@type='simplistic']">
      <xsl:apply-templates/>
</xsl:template>
<xsl:template match="tei:def">
      <xsl:apply-templates/>.
</xsl:template>
<xsl:template match="tei:note[@type='in-vocabulary-of']">
   <xsl:choose>
   <xsl:when test=". = 'athenaze'">
      <xsl:text>A </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'athenaze1'">
      <xsl:text>A1 </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'athenaze2'">
      <xsl:text>A2 </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'cheadle1'">
      <xsl:text>C1 </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'cheadle2'">
      <xsl:text>C2 </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'hansen and quinn'">
      <xsl:text>HQ </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'luschnig1'">
      <xsl:text>L1 </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'mastronarde'">
      <xsl:text>M </xsl:text>
   </xsl:when>
   <xsl:when test=". = 'xenophon anabasis'">
      <xsl:text>XA </xsl:text>
   </xsl:when>
   </xsl:choose>
</xsl:template>
<xsl:template match="tei:note[@type='language-or-dialect']">
</xsl:template>

<!-- end: Lexical Entries -->

<!-- note: "code" is phrase-level, 
           for block-level code examples, use 'ab type="code"' -->
<xsl:template match="tei:code">
   <xsl:text>&#xA;</xsl:text>
   <tt>
   <xsl:apply-templates/>
   </tt>
</xsl:template>

<xsl:template match="tei:distinct[@type='filename']">
   <xsl:text>&#xA;</xsl:text>
   <tt>
   <xsl:apply-templates/>
   </tt>
</xsl:template>

<xsl:template match="tei:distinct[@type='input']">
   <xsl:text>&#xA;</xsl:text>
   <tt>
   <xsl:apply-templates/>
   </tt>
</xsl:template>

<xsl:template match="tei:distinct[@type='output']">
   <xsl:text>&#xA;</xsl:text>
   <tt>
   <xsl:apply-templates/>
   </tt>
</xsl:template>

<xsl:template match="tei:distinct[@type='output-block']">
   <xsl:text>&#xA;</xsl:text>
   <pre>
   <xsl:apply-templates/>
   </pre>
</xsl:template>

<xsl:template match="tei:distinct[@type='keyword']">
   <xsl:text>&#xA;</xsl:text>
   <b>
   <xsl:apply-templates/>
   </b>
</xsl:template>

<xsl:template match="tei:distinct[@type='variable']">
   <xsl:text>&#xA;</xsl:text>
   <tt>
   <xsl:apply-templates/>
   </tt>
</xsl:template>

<!-- "function" is a synonym for "variable" in usage here -->
<xsl:template match="tei:distinct[@type='function']">
   <xsl:text>&#xA;</xsl:text>
   <u><tt>
   <xsl:apply-templates/>
   </tt></u>
</xsl:template>

<xsl:template match="tei:distinct[@type='program']">
   <xsl:text>&#xA;</xsl:text>
   <b><tt>
   <xsl:apply-templates/>
   </tt></b>
</xsl:template>

<xsl:template match="tei:figure">
   <xsl:text>&#xA;</xsl:text>
   <xsl:apply-templates/>
   <p><xsl:value-of select="tei:head" /></p>
</xsl:template>

<xsl:template match="tei:figDesc">
</xsl:template>

<!--
Case 1: non-scalable bitmap
        if width exists 
           assume height exists, too
           assume that this is a non-scalable bitmap
Case 2: not a non-scalable bitmap
           generate bitmap img tag
              check first for PNG, then for JPG
              For now, don't specify width/height

              I can't figure out how to extract width/height from the image
              from XSLT.  The shell commands to get the information are:
echo `identify linking-example.png` |  awk '{ print substr($3,0,index($3,"x")-1) }'
echo `identify linking-example.png` |  awk '{ print substr($3,index($3,"x")+1,length($3)-index($3,"x")) }'

           check if SVG version exists; generate href to it if it does
-->
<xsl:template match="tei:graphic">
   <xsl:variable name="image-name-base"
                 select="@url" />
   <xsl:choose>
      <xsl:when test="@width">
         <img>
            <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-current" /><xsl:value-of select="@url" /></xsl:attribute>
<!-- do not generate these here; will be patched up using awk in makefile
            <xsl:attribute name="width">
                <xsl:value-of select="@width" /></xsl:attribute>
            <xsl:attribute name="height">
                <xsl:value-of select="@height" /></xsl:attribute>
-->
            <xsl:attribute name="alt">
                <xsl:value-of select="../tei:figDesc" /></xsl:attribute>
         </img>
      </xsl:when>
      <xsl:otherwise>
         <img>
            <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-current" /><xsl:value-of select="$image-name-base" />-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
            <xsl:attribute name="alt">image <xsl:value-of select="$image-name-base" />-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
         </img>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<!-- =============================== -->
<!-- Minimalist Literate Programming -->
<!-- =============================== -->
<!-- Note: The run-on line in the PRE definition is important for avoiding
           extra space in the HTML. -->
<xsl:template match="tei:ab[@type='code-chunk-start']">
   <xsl:text>&#xA;</xsl:text>
   <pre class="code-chunk">&lt;&lt;<xsl:value-of select="@xml:id" />&gt;&gt;=<xsl:apply-templates/></pre>
</xsl:template>
<xsl:template match="tei:ab[@type='code-chunk']">
   <xsl:text>&#xA;</xsl:text>
   <pre class="code-chunk">&lt;&lt;<xsl:value-of select="@xml:id" />&gt;&gt;=<xsl:apply-templates/></pre>
</xsl:template>

<xsl:template match="tei:seg[@type='code-chunk-ref']">
&lt;&lt;<u><xsl:apply-templates/></u>&gt;&gt;</xsl:template>

<xsl:template match="tei:ab[@type='gmlp-static-reference']">
   <xsl:apply-templates/>
</xsl:template>

<!-- ============= -->
<!-- Legal Section -->
<!-- ============= -->
<xsl:template match="tei:ab[@type='copyright']">
   <xsl:text>&#xA;</xsl:text>
   <p>
   <xsl:text>All portions of this document not noted otherwise are Copyright &#xA9; </xsl:text>
   <xsl:apply-templates/>
   </p>
</xsl:template>

<!-- ========================== -->
<!-- Backward and Forward Links -->
<!-- ========================== -->
<!-- These differ from the "Standard Links" below in that they require
     information coded in the TEI. -->
<xsl:template match="tei:div[@type='backward-forward-links']">
   <xsl:text>&#xA;</xsl:text>
   <p>
   <xsl:apply-templates/>
   </p>
</xsl:template>
<xsl:template match="tei:ab[@type='backward']">
   <xsl:text>&#xA;</xsl:text>
   §<a>
   <xsl:attribute name="href">
                <xsl:value-of select="@n" />-<xsl:value-of select="$scale-factor" />.html</xsl:attribute>
   Back to Previous Section 
   </a>§
</xsl:template>
<xsl:template match="tei:ab[@type='forward']">
   <xsl:text>&#xA;</xsl:text>
   <a>
   <xsl:attribute name="href">
                <xsl:value-of select="@n" />-<xsl:value-of select="$scale-factor" />.html</xsl:attribute>
   Forward to Next Section
   </a>§
</xsl:template>

<!-- ============== -->
<!-- Standard Links -->
<!-- ============== -->
<!-- (Automatically generated, not from TEI encoding per se.) -->

<!-- Aside on CSS:
   I'm hiding a bit of CSS trickery here.  Originally, I'd wanted to
   put the "Home", "Category", etc. words below the images.  
   The trouble with that is that, first, CSS doesn't automatically
   make the enclosing div as high as the divs it encloses - so it
   ends up here being just an empty invisible line with the various
   link divs "hanging" down from it.  I can overcome this by outputting
   an empty div after the "Up" div and specifying in its CSS
   (either here with "style=" or in the CSS stylesheet) "clear: both".
   But although this makes the enclosing div actually enclose the
   link divs, it still isn't possible to make the link divs align
   to the bottom of the enclosing div (I *think* they still think
   it's a zero-height line, but I could be wrong here).

   So I avoid the problem entirely by putting the words above the
   pictures, so that top-alignment seems natural.

   There must be a philosophical message here, but whatever it is
   I don't think I like it.
-->

<xsl:template name="standard-links">
   <xsl:text>&#xA;</xsl:text>
   <hr />
   <div class="standard-links">

   <!-- note: must use the lt entity 
              rather than the literal left-angle character -->
   <!-- Home link appears in locations 2, 1, 0 -->
   <xsl:if test="$location &lt; 3">

      <xsl:variable name="home-relative-url">
         <xsl:call-template name="linkup">
            <xsl:with-param name="levels" select="$depth-in-hierarchy"/>
         </xsl:call-template>
      </xsl:variable>

      <div class="standard-linking-image">
      <a>
      <xsl:attribute name="href"><xsl:value-of select="$home-relative-url" />index.html</xsl:attribute>
      Home<br />
<!-- 
NOTE: 
1. Originally the image link was coded to refer to a copy of the linking image
   for "Home" in the current directory.  Thus:

         <xsl:attribute name="src">link-home-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

   The images are still replicated into the current directories in the build
   tree, so I can always go back to this.  However, it proved space-inefficient
   to replicate these images in the tree on the server.

2. Then I tried simply referring up to a single copy of the "Home" image in
   the document root directory.  Thus:

         <xsl:attribute name="src"><xsl:value-of select="$home-relative-url" />link-home-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

3  This was more space-efficient.  However, I've ended up having to gather
   all of the images in a duplicate tree from a separate root so as to be
   able to serve them, if necessary, from a different server.  It wasn't
   stricly necessary to do this with the "Home" image (it's just one image),
   but was cleaner to be consistent.  Interstingly, since /images is a single
   location and the Home image is a single image, it is simpler. Thus:

         <xsl:attribute name="src">/images/link-home-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

-->
      <img>
         <xsl:attribute name="src">/images/link-home-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
         <xsl:attribute name="alt">Link to Home Page</xsl:attribute>
      </img>
      </a>
      </div>
   </xsl:if>

   <!-- Category link appears in locations 1, 0 -->
   <xsl:if test="$location &lt; 2">

<!--
      <xsl:if test="$own-basename = 'index'">
-->

            <xsl:variable name="category-relative-url">
            <xsl:call-template name="linkup">
               <xsl:with-param name="levels" select="$depth-in-category"/>
            </xsl:call-template>
         </xsl:variable>

         <div class="standard-linking-image">
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$category-relative-url" />index.html</xsl:attribute>
         Category<br />
         <img>
<!-- 
NOTE: 
1. Originally the image link was coded to refer to a copy of the linking image
   for "Category" in the current directory.  Thus:

         <xsl:attribute name="src">link-category-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

   The images are still replicated into the current directories in the build
   tree, so I can always go back to this.  However, it proved space-inefficient
   to replicate these images in the tree on the server.

2. Then I tried simply referring up to a single copy of the "Category" image in
   the document root directory.  Thus:

         <xsl:attribute name="src"><xsl:value-of select="$category-relative-url" />link-category-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

   SUBNOTE: If this were a link to the category page on the current page,
            it would be link-category-sfN (as originally coded, with the
            link-category-sfN files propagaged; this propagation is still done).
            But now, to save space, I'm linking up the the category top level.
            On THAT page, the "category" is the "topic," so this is
            LINK/UP/TO/CATEGORY/START/link-topic-sfN

3  This was more space-efficient.  However, I've ended up having to gather
   all of the images in a duplicate tree from a separate root so as to be
   able to serve them, if necessary, from a different server. Thus:

            <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-category-top" />link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
-->
            <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-category-top" />link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
            <xsl:attribute name="alt">Link to Category Page</xsl:attribute>
         </img>
         </a>
         </div>
      </xsl:if>

<!--
      <xsl:if test="$own-basename != 'index'">
         <div class="standard-linking-image">
         <a href="index.html">
         <img>
            <xsl:attribute name="src">link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
            <xsl:attribute name="alt">Link to Category Page at same level; = Topic</xsl:attribute>
         </img><br />
         Category
         </a>
         </div>
      </xsl:if>

   </xsl:if>
-->

<!-- 
NOTE: 
1. Originally the image link was coded to refer to a copy of the linking image
   for "Topic" in the current directory.  Thus:

         <xsl:attribute name="src">link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

   The images are still replicated into the current directories in the build
   tree, so I can always go back to this.  However, it proved space-inefficient
   to replicate these images in the tree on the server.

2. [actually, I forgot the "2" stage of this and fixed it on "3"

3. I've gathered all of the images in a duplicate tree from a separate root
   able to serve them, if necessary, from a different server. Thus:
-->
   <!-- Topic link appears in location 0 -->
   <xsl:if test="$location = 0">
      <div class="standard-linking-image">
      <a href="index.html">
      Topic<br />
      <img>
         <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-current" />link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
         <xsl:attribute name="alt">Link to Topic Page</xsl:attribute>
      </img>
      </a>
      </div>
   </xsl:if>

   <!-- Up link appears in locations 1, 0 -->
   <xsl:if test="$location &lt; 2">
      <div class="standard-linking-image">
      <a href="../index.html">
      Up<br />
      <img>
<!-- 
NOTE: 
1. Originally the image link was coded to refer to a copy of the linking image
   for "Up" in the current directory.  Thus:

         <xsl:attribute name="src">link-up-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

   The images are still replicated into the current directories in the build
   tree, so I can always go back to this.  However, it proved space-inefficient
   to replicate these images in the tree on the server.

2. Then I tried simply referring up to a single copy of the "Up" image in
   the document root directory.  Thus:

         <xsl:attribute name="src">../link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>


   SUBNOTE: If this were a link to the category page on the current page,
            it would be link-up-sfN (as originally coded, with the
            link-category-sfN files propagaged; this propagation is still done).
            But now, to save space, I'm linking up the the category top level.
            On THAT page, the "category" is the "topic," so this is
            LINK/UP/TO/CATEGORY/START/link-topic-sfN

3  This was more space-efficient.  However, I've ended up having to gather
   all of the images in a duplicate tree from a separate root so as to be
   able to serve them, if necessary, from a different server.  Thus:

            <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-current" />../link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>

-->
            <xsl:attribute name="src">/images/<xsl:value-of select="$path-to-current" />../link-topic-sf<xsl:value-of select="$scale-factor" />.jpg</xsl:attribute>
         <xsl:attribute name="alt">Link Up</xsl:attribute>
      </img>
      </a>
      </div>
   </xsl:if>

   </div>

   <!-- "Select Resolution: ..." -->
   <div class="broadside">

   <xsl:variable name="home-relative-url">
      <xsl:call-template name="linkup">
         <xsl:with-param name="levels" select="$depth-in-hierarchy"/>
      </xsl:call-template>
   </xsl:variable>

<!-- this is a kludge, but I'm tired :-)
     I should loop up to the max scale factor -->
<!-- and another kludge for now; just make resolutions 0  -->
   <xsl:if test="$scale-factor = 0">
      <p>Select
         <a>
<!-- HACK: temporarily remove the -sfN
         <xsl:attribute name="href"><xsl:value-of select="$home-relative-url" />about-resolution-0.html</xsl:attribute>Resolution</a>:
-->
         <xsl:attribute name="href"><xsl:value-of select="$home-relative-url" />about-resolution.html</xsl:attribute>Resolution</a>:
         <xsl:text>0  [other resolutions temporarily disabled due to lack of disk space]</xsl:text>
<!--
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$own-basename" />-1.html</xsl:attribute>1 </a>
-->
<!--
         <xsl:text>  </xsl:text>
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$own-basename" />-2.html</xsl:attribute>2 </a>
-->
      </p>
   </xsl:if>
   <xsl:if test="$scale-factor = 1">
      <p>Select
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$home-relative-url" />about-resolution-1.html</xsl:attribute>Resolution</a>:
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$own-basename" />-0.html</xsl:attribute>0 </a>
         <xsl:text>   1  </xsl:text>
<!--
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$own-basename" />-2.html</xsl:attribute>2 </a>
-->
      </p>
   </xsl:if>
<!--
   <xsl:if test="$scale-factor = 2">
      <p>Select
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$home-relative-url" />about-resolution-2.html</xsl:attribute>Resolution</a>:
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$own-basename" />-0.html</xsl:attribute>0 </a>
         <xsl:text>  </xsl:text>
         <a>
         <xsl:attribute name="href"><xsl:value-of select="$own-basename" />-1.html</xsl:attribute>1 </a>
         <xsl:text>  2</xsl:text>
      </p>
   </xsl:if>
-->

<!-- W3C validators -->
<!-- If the objects, the W3C validator logo images, can be loaded from the
     W3C's server, then it is likely that their links also work.
     If they cannot, then keep their links, but default to text messages
     that indicate that validation is not available.
     This is useful whenever there is no connection to the public Internet. -->
<!--
<p>
<a   href="http://validator.w3.org/check?uri=referer">
<object   title="W3C XHTML 1.0 Validated Logo"
           type="image/png" width="88" height="31"
        standby="W3C XHTML 1.0 Validated Logo"
           data="http://www.w3.org/Icons/valid-xhtml10.png">
   [XHTML 1.0 Validation Unavailable]
</object></a>
<a   href="http://jigsaw.w3.org/css-validator/check/referer">
<object   title="W3C CSS Validated Logo"
           type="image/png" width="88" height="31"
        standby="W3C CSS Validated Logo" 
           data="http://jigsaw.w3.org/css-validator/images/vcss.png">
   [CSS Validation Unavailable]
</object></a>
</p>
-->


   </div>
</xsl:template>

<!-- print out n "../"s, for use in standard linking -->
<!-- use recursion to do loops in XSL -->
<xsl:template name="linkup">
   <xsl:param name="levels" select="1"/>
   <xsl:if test="$levels > 0">
      <xsl:text>../</xsl:text>
      <xsl:call-template name="linkup">
         <xsl:with-param name="levels" select="$levels -1"/>
      </xsl:call-template>
   </xsl:if>
</xsl:template>
<!-- "linkup-minus-one" is useful for getting at things at the
     directory level one down from the start of a category
     (things such as the "./lexicon" subdirectory of a language Notebook). -->
<xsl:template name="linkup-minus-one">
   <xsl:param name="levels" select="1"/>
   <xsl:if test="$levels > 1">
      <xsl:text>../</xsl:text>
      <xsl:call-template name="linkup">
         <xsl:with-param name="levels" select="$levels -1"/>
      </xsl:call-template>
   </xsl:if>
</xsl:template>



<!-- =============== -->
<!-- TEI Back Matter -->
<!-- =============== -->
<xsl:template match="tei:back">
   <xsl:text>&#xA;</xsl:text>
   <xsl:call-template name="legal-matters" />
   <xsl:apply-templates/>
   <xsl:text>&#xA;</xsl:text>
   <xsl:call-template name="standard-links" />
</xsl:template>


<xsl:template match="tei:ab[@type='about-the-images-link']">
   <xsl:text>&#xA;</xsl:text>
   <div class="broadside">
   <p>
   <a>
<!-- HACK: temporarily remove the -sfN
      <xsl:attribute name="href">about-the-images-<xsl:value-of select="$scale-factor" />.html</xsl:attribute>About the images</a></p>
-->
      <xsl:attribute name="href">about-the-images.html</xsl:attribute>About the images</a></p>
   </div>
</xsl:template>

</xsl:stylesheet>

